%% Load variables

clear variables;
close all;
clc;

IMU = table2array(readtable('IMU.csv'));
dt_IMU = 1000;
lastImuIdx = 0;

GPS = table2array(readtable('GPS.csv'));
dt_GPS = 200000;
lastGPSIdx = 0;

baro = table2array(readtable('barometer.csv'));
dt_baro = mean(diff(baro(:, 1)));
lastBaroIdx = 0;

t_end = min([IMU(end,1), baro(end,1), GPS(end,1)]);

%% Simulation
% IF YOU NEED TO EDIT THIS FILE, EDIT THIS SECTION ONLY

% Init simulation variables
data = [];
flag_apogee = 0;
apogee_time = -1;

% Simulation
for t = 0:2000:t_end

    % Getting measurment from sensors
    baro_idx = sum(baro(:,1) <= t);
    if baro_idx > lastBaroIdx
        pressure_measurment = baro(baro_idx, 2);
        lastBaroIdx = baro_idx;
    else
        pressure_measurment = baro(lastBaroIdx,2);
    end

    imu_idx = sum(IMU(:,1) <= t);
    if imu_idx > lastImuIdx
        IMU_measurment = IMU(imu_idx, 2);
        lastImuIdx = imu_idx;
    else
        IMU_measurment = IMU(lastImuIdx,2:7);
    end

    gps_idx = sum(GPS(:,1) <= t);
    if gps_idx > lastGPSIdx
        GPS_measurment = GPS(gps_idx, 2:4);
        lastGPSIdx = gps_idx;
    else
        GPS_measurment = GPS(lastGPSIdx,2:4);
    end
    
    % Apogee detection
    [flag_apogee, data] = apogee_detector(pressure_measurment, IMU_measurment, GPS_measurment, data);
    
    % Save first apogee detection time
    if flag_apogee && (apogee_time < 0)
        apogee_time = t;
    end

end

%% Plot Result

% check that the apogee detector did something
if apogee_time < 0 
    error("The apogee time is negative")
end

lift_off_time = 7.4; %[s]
landing_time = 137.4060; %[s]

% Plot baro
figure;
plot(baro(:,1)/1e6, baro(:,2) ); 
grid minor; xlabel('time [s]'); ylabel('Pa');
xline(apogee_time/1e6, '--', "apogee");
xline(lift_off_time, '-', "lift off");
xline(landing_time, '-', "landing");
title('Barometer')

% Plot accelerometer
figure();
subplot(3,1,1);
plot(IMU(:,1)/1e6, IMU(:,2) ); 
grid minor; xlabel('time [s]'); ylabel('m/s^2');
xline(apogee_time/1e6, '--', "apogee");
xline(lift_off_time, '-', "lift off");
xline(landing_time, '-', "landing");
title('Accelerometer X')

subplot(3,1,2);
plot(IMU(:,1)/1e6, IMU(:,3) ); 
grid minor; xlabel('time [s]'); ylabel('m/s^2');
xline(apogee_time/1e6, '--', "apogee");
xline(lift_off_time, '-', "lift off");
xline(landing_time, '-', "landing");
title('Accelerometer Y')

subplot(3,1,3);
plot(IMU(:,1)/1e6, IMU(:,4) ); 
grid minor; xlabel('time [s]'); ylabel('m/s^2');
xline(apogee_time/1e6, '--', "apogee");
xline(lift_off_time, '-', "lift off");
xline(landing_time, '-', "landing");
title('Accelerometer Z')

% Plot GPS
figure();
plot(GPS(:,1)/1e6, GPS(:,4) ); 
grid minor; xlabel('time [s]'); ylabel('m');
xline(apogee_time/1e6, '--', "apogee");
xline(lift_off_time, '-', "lift off");
xline(landing_time, '-', "landing");
title('GPS height')
